"""Minecraft Curse CLI Client"""

import gettext
from pathlib import Path

#: Consistent names definitions
RESOURCE_NAME = __package__

#: Root of the package
pkgdir = Path(__file__).resolve().parent

#: Root of the locale files
localedir = pkgdir/'locales'

#: Translation machinery for the app
translation = gettext.translation(
    domain=__package__,
    # Allow the locale files to be stored in system folder
    localedir=str(localedir) if localedir.is_dir() else None,
    fallback=True,
)
_ = translation.gettext



"""Tests for the curse submodule."""


import bz2
import datetime
import json
from pathlib import Path

import betamax
import ijson
import pytest
import requests
import responses

from sqlalchemy.orm.session import Session as SQLSession

from mccurse import curse


# Fixtures

@pytest.fixture
def minecraft_feed() -> curse.Feed:
    """Feed for testing, with session"""

    return curse.Feed(game_id=432, session=requests.Session())


@pytest.fixture
def file_database(tmpdir) -> curse.Database:
    """Database potentially located in temp dir."""

    return curse.Database('test', Path(str(tmpdir)))


@pytest.fixture
def filled_database(file_database) -> curse.Database:
    """Database with some mods filled in."""

    # Create structure
    curse.AddonBase.metadata.create_all(file_database.engine)

    # Add few mods
    session = SQLSession(bind=file_database.engine)
    session.add_all([
        curse.Mod(id=42, name='tested', summary="Mod under test"),
        curse.Mod(id=45, name='tester', summary="Validate tested mod"),
        curse.Mod(id=3, name='unrelated', summary="Dummy"),
    ])
    session.commit()

    return file_database


@pytest.fixture
def game(tmpdir) -> curse.Game:
    """Game pre-initialized with testing objects."""

    # leave session intact
    return curse.Game(
        id=432,  # Minecraft id
        name='Minecraft',
        version='1.10.2',
        cache_dir=Path(str(tmpdir)),
    )


# Feed tests

@responses.activate
def test_complete_feed_url(minecraft_feed):
    """Generate correct feed URL?"""

    EXPECT = '/'.join((
        curse.Feed._BASEURL.format(id=minecraft_feed.game_id),
        curse.Feed._COMPLETE_URL,
    ))

    assert minecraft_feed.complete_url == EXPECT
    assert len(responses.calls) == 0


@responses.activate
def test_complete_timestamp_url(minecraft_feed):
    """Generate correct timestamp URL?"""

    EXPECT = '/'.join((
        curse.Feed._BASEURL.format(id=minecraft_feed.game_id),
        curse.Feed._COMPLETE_URL + '.txt',
    ))

    assert minecraft_feed.complete_timestamp_url == EXPECT
    assert len(responses.calls) == 0


@responses.activate
def test_content_decoding(minecraft_feed):
    """Decode the stream contents correctly?"""

    EXPECT = 'Ahoj světe'
    INPUT = bz2.compress(EXPECT.encode('utf-8'))

    with minecraft_feed._decode_contents(INPUT) as stream:
        decoded = stream.read()

    assert decoded == EXPECT
    assert len(responses.calls) == 0


@responses.activate
def test_timestamp_decoding(minecraft_feed):
    """Decode the timestamp contents correctly?"""

    EXPECT = datetime.datetime(
        year=2012, month=8, day=2,
        hour=12, minute=0, microsecond=123000,
        tzinfo=datetime.timezone.utc,
    )
    # Input is timestamp in microseconds
    INPUT = int(EXPECT.timestamp()*1000)

    assert minecraft_feed._decode_timestamp(INPUT) == EXPECT
    assert len(responses.calls) == 0


def test_fetch_complete_timestamp(minecraft_feed):
    """Fetches and decodes the timestamp correctly?"""

    with betamax.Betamax(minecraft_feed.session).use_cassette('feed-timestamp'):  # noqa: E501
        timestamp = minecraft_feed.fetch_complete_timestamp()
        assert isinstance(timestamp, datetime.datetime)


def test_fetch_complete(minecraft_feed):
    """Fetches and decodes the contents correctly?"""

    with betamax.Betamax(minecraft_feed.session).use_cassette('fetch-feed'), \
            minecraft_feed.fetch_complete() as feed:
        timestamp = next(ijson.items(feed, 'timestamp'), None)
        assert isinstance(timestamp, int)


# Database tests

def test_file_uri(file_database):
    """Construct filesystem URI correctly?"""

    EXPECT = '{scheme}/{path!s}/{name}'.format(
        scheme=curse.Database._SCHEME,
        path=file_database.root_dir.resolve(),
        name=curse.Database._BASENAME.format(
            game_name=file_database.game_name
        ),
    )

    assert file_database.uri == EXPECT


def test_engine_creation(file_database):
    """Is the DB engine created correctly in both types of DB?"""

    QUERY = 'SELECT 1+1'
    EXPECT = (2,)

    assert file_database.engine.execute(QUERY).first() == EXPECT


def test_database_versioning(file_database):
    """Is the data version persisted correctly?"""

    INPUT = datetime.datetime(2012, 12, 12, tzinfo=datetime.timezone.utc)
    timestamp = int(INPUT.timestamp())

    file_database.version = INPUT

    dbstamp, = file_database.engine.execute('PRAGMA user_version').first()

    assert dbstamp == timestamp
    assert file_database.version == INPUT


# Mod tests

def test_json_parsing():
    """Is the mod correctly constructed from JSON data?"""

    INPUT = {
      "Id": 74072,
      "Name": "Tinkers Construct",
      "Summary": "Modify all the things, then do it again!",
    }
    EXPECT = curse.Mod(
        id=74072,
        name="Tinkers Construct",
        summary="Modify all the things, then do it again!",
    )

    assert curse.Mod.from_json(INPUT) == EXPECT


def test_mod_search(filled_database):
    """Does the search return expected results?"""

    EXPECT_IDS = {42, 45}

    session = SQLSession(bind=filled_database.engine)
    selected = curse.Mod.search(session, 'Tested')

    assert {int(m.id) for m in selected} == EXPECT_IDS


def test_mod_find(filled_database):
    """Does the search find the correct mod or report correct error?"""

    session = SQLSession(bind=filled_database.engine)

    assert curse.Mod.find(session, 'Tested').id == 42

    with pytest.raises(curse.MultipleResultsFound):
        curse.Mod.find(session, 'test')

    with pytest.raises(curse.NoResultFound):
        curse.Mod.find(session, 'nonsense')


# Game tests

@responses.activate
def test_gamedata_refresh(game):
    """Does the game refreshes its data correctly?"""

    now = datetime.datetime.now(tz=datetime.timezone.utc).replace(microsecond=0)  # noqa: E501
    curse_timestamp = int(now.timestamp()*1000)

    # Mock project feed
    mod_path = {'CategorySection': {'Path': 'mods'}}
    other_path = {'CategorySection': {'Path': 'other'}}
    mock_feed_body = {
        'timestamp': curse_timestamp,
        'data': [
            dict(mod_path, Name='test', Id=42, Summary='Test mod'),
            dict(mod_path, Name='nott', Id=15, Summary='No test!'),
            dict(mod_path, Name='tinker', Id=432, Summary='Metamod'),

            dict(other_path, Name='map', Id=16, Summary='Map pack'),
        ]
    }

    # Complete feed
    responses.add(
        responses.GET,
        game.feed.complete_url,
        body=bz2.compress(json.dumps(mock_feed_body).encode('utf-8')),
    )
    # Timestamp
    responses.add(
        responses.GET,
        game.feed.complete_timestamp_url,
        body=str(curse_timestamp).encode('utf-8'),
    )

    game.refresh_data()
    sess = game.database.session()

    assert len(responses.calls) == 2
    assert game.database.version == now
    assert sess.query(curse.Mod).count() == len([
        d for d in mock_feed_body['data']
        if d['CategorySection']['Path'] == 'mods'
    ])


@responses.activate
def test_gamedata_fresh(game):
    """Does the game check the data validity correctly?"""

    data_timestamp = datetime.datetime(
        2017, 1, 15, 0, 0, 0,
        tzinfo=datetime.timezone.utc,
    )
    game.database.version = data_timestamp

    assert game.have_fresh_data(
        valid_period=datetime.timedelta(hours=24),
        now=data_timestamp+datetime.timedelta(hours=12),
    )
    assert not game.have_fresh_data(
        valid_period=datetime.timedelta(hours=24),
        now=data_timestamp+datetime.timedelta(hours=24),
    )



"""Tests for the proxy submodule"""

from io import StringIO
from functools import partial

import attr
import pytest
import requests
import responses
import yaml

try:
    from yaml import CLoader as YAMLLoader, CDumper as YAMLDumper
except ImportError:
    from yaml import Loader as YAMLLoader, Dumper as YAMLDumper

from mccurse import proxy


# Helper functions

yaml_dump = partial(yaml.dump, Dumper=YAMLDumper)
yaml_load = partial(yaml.load, Loader=YAMLLoader)


# Fixtures

@pytest.fixture
def dummy_auth() -> proxy.Authorization:
    return proxy.Authorization(
        user_id=42,
        token='token',
    )


# Authorization tests

@responses.activate
def test_modified_request(dummy_auth):
    """Is the request properly modified?"""

    # Expected header
    EXPECT = 'Token {}:{}'.format(dummy_auth.user_id, dummy_auth.token)

    url = 'https://example.com'

    responses.add(responses.GET, url, json=[])

    requests.get(url, auth=dummy_auth)

    assert len(responses.calls) == 1

    request = responses.calls[0].request
    assert request.headers['Authorization'] == EXPECT


@responses.activate
def test_authorization_login(dummy_auth):
    """Is the authorization properly constructed from the proxy response?"""

    EXPECT = dummy_auth

    url = '/'.join((proxy.HOME_URL, 'authenticate'))
    responses.add(responses.POST, url, json={
        'session': {
            'user_id': EXPECT.user_id,
            'token': EXPECT.token,
        },
    })

    auth = proxy.Authorization.login('user', 'pass')

    assert EXPECT == auth


def test_auth_loading(dummy_auth):
    """Is the authentication properly loaded from file?"""

    correct = StringIO(yaml_dump(attr.asdict(dummy_auth)))
    empty = StringIO()

    assert proxy.Authorization.load(correct) == dummy_auth

    with pytest.raises(ValueError):
        proxy.Authorization.load(empty)


def test_auth_store(dummy_auth):
    """Is the authentication properly stored into a file?"""

    buffer = StringIO()
    dummy_auth.dump(buffer)

    data = yaml_load(buffer.getvalue())

    assert data == attr.asdict(dummy_auth)



"""Various utilities and language enhancements."""


from pathlib import Path

import requests
import xdg.BaseDirectory

from .. import RESOURCE_NAME


# Filesystem standard directories
def default_cache_dir(directory: Path = None) -> Path:
    """Get default cache dir if directory is None,
    else return directory as it is.
    """

    if directory is None:
        return Path(xdg.BaseDirectory.save_cache_path(RESOURCE_NAME))
    else:
        return directory


def default_data_dir(directory: Path = None) -> Path:
    """Get default data dir if directory is None,
    else return directory as it is.
    """

    if directory is None:
        return Path(xdg.BaseDirectory.save_data_path(RESOURCE_NAME))
    else:
        return directory


def default_new_session(session: requests.Session = None) -> requests.Session:
    """Create new Requests' Session, if none is provided.
    Otherwise, return session as it is.
    """

    if session is None:
        return requests.Session()
    else:
        return session



"""Custom wrappers, validators and coercers for the cerberus library."""

from datetime import datetime
from enum import Enum
from typing import Any, Callable, Type, Union

from iso8601 import parse_date


# Named generic types
ErrorCallback = Callable[[Any, str], None]


# Custom validators
def instance_of(cls: Type) -> Callable[[Any, Any, ErrorCallback], bool]:
    """Create validator for an arbitrary type.

    Keyword arguments:
        cls: The type to check the instance against.

    Returns:
        Validator callable.
    """

    def validate(field: Any, value: Any, error: ErrorCallback) -> bool:
        if isinstance(value, cls):
            return True
        else:
            msg = "Value '{!s}' is not of type '{!s}'".format(
                    value, cls.__name__,
            )
            error(field, msg)
            return False

    return validate


# Custom coercers
def isodate(value: Union[str, datetime]) -> datetime:
    """Coerce ISO-8601 date string to `class`:datetime:, if needed.

    Keyword arguments:
        value: The string to be coerced. Alternatively, if the value is
            already `class`:datetime:, pass it unchanged.

    Returns:
        Coerced value.
    """

    if isinstance(value, str):
        return parse_date(value)
    else:  # Do not perform validation, it is different task
        return value


def fromname(cls: Type[Enum]) -> Callable[[Union[str, Enum]], Enum]:
    """Create coercer for an arbitrary `class`:Enum: from its possible names.

    Keyword arguments:
        cls: The enumeration to coerce to.

    Returns:
        Coercer which converts string values to appropriate enumeration,
        or pass the value unchanged.
    """

    def coerce(value: Union[str, Enum]) -> Enum:
        if isinstance(value, str):
            return cls[value]
        else:
            return value

    return coerce



"""Tests for the pack submodule"""

from copy import deepcopy
from datetime import datetime
from io import StringIO

import cerberus
import pytest

from mccurse import pack
from mccurse.curse import Game
from mccurse.util import yaml


@pytest.fixture
def pack_validator() -> cerberus.Validator:
    return cerberus.Validator(cerberus.schema_registry.get('pack'))


@pytest.fixture
def valid_mod_file() -> dict:
    return {
        'id': '42',
        'name': 'test-mod-file.jar',
        'date': '2017-01-24T18:01+01:00',
        'release': 'Beta',
    }


@pytest.fixture
def invalid_mod_file() -> dict:
    return {
        'name': 'testfile.zip',
        'date': 'yesterday',
        'release': 'Released',
        'deps': [123, 456],
    }


@pytest.fixture
def empty_mod() -> dict:
    """Empty (without file) mod entry"""
    return {
        'id': 74072,
        'name': 'Tinkers Contruct',
    }


@pytest.fixture
def valid_mod(empty_mod, valid_mod_file) -> dict:
    return dict(**empty_mod, file=deepcopy(valid_mod_file))


@pytest.fixture
def invalid_mod(empty_mod, invalid_mod_file) -> dict:
    return dict(**empty_mod, file=invalid_mod_file)


@pytest.fixture
def minimal_pack() -> dict:
    return {
        'game': {'version': '1.10.2'},
    }


@pytest.fixture
def valid_pack(valid_mod) -> dict:
    return {
        'game': {
            'name': 'Minecraft',
            'version': '1.10.2',
        },
        'mods': [deepcopy(valid_mod)],
        'dependencies': [],
    }


@pytest.fixture
def invalid_pack(invalid_mod) -> dict:
    return {
        'game': {
            'name': 'Minecraft',
            'version': '1.10.2',
        },
        'mods': [deepcopy(invalid_mod)],
        'dependencies': [],
    }


@pytest.fixture
def valid_yaml(valid_pack) -> StringIO:
    stream = StringIO()
    yaml.dump(valid_pack, stream)
    stream.seek(0)

    return stream


@pytest.fixture
def invalid_yaml(invalid_pack) -> StringIO:
    stream = StringIO()
    yaml.dump(invalid_pack, stream)
    stream.seek(0)

    return stream


def test_release():
    """Test release creation and ordering"""

    A = pack.Release['Alpha']
    B = pack.Release['Beta']
    R = pack.Release['Release']

    assert A == pack.Release['Alpha']
    assert A < B < R
    assert R > B > A
    assert A != B


def test_release_and_yaml():
    """Serialization of Release to YAML works as intended?"""

    data = [pack.Release['Alpha']]
    text = '- Alpha\n'

    assert yaml.dump(data) == text
    assert yaml.load(text) == data


def test_mod_file_schema(valid_mod_file, invalid_mod_file):
    """Mod file schema bahving as expected?"""

    schema = cerberus.schema_registry.get('mod-file')
    vld = cerberus.Validator(schema)
    inv = cerberus.Validator(schema)

    assert vld.validate(valid_mod_file)
    assert isinstance(vld.document['id'], int)
    date = vld.document['date']
    assert isinstance(date, datetime) and date.tzinfo is not None
    assert isinstance(vld.document['release'], pack.Release)

    assert not inv.validate(invalid_mod_file)
    assert inv.errors


def test_mod_schema(valid_mod, invalid_mod):
    """Mod schema behaving as expected?"""

    schema = cerberus.schema_registry.get('mod')
    vld = cerberus.Validator(schema)
    inv = cerberus.Validator(schema)

    assert vld.validate(valid_mod)
    assert isinstance(vld.document['id'], int)
    assert isinstance(vld.document['file']['id'], int)

    assert not inv.validate(invalid_mod)


def test_pack_schema(minimal_pack, valid_pack, invalid_pack):
    """Pack schema behaving as expected?"""

    schema = cerberus.schema_registry.get('pack')
    minimal, valid, invalid = map(cerberus.Validator, [schema]*3)

    assert minimal.validate(minimal_pack)
    assert valid.validate(valid_pack)
    assert not invalid.validate(invalid_pack)


def test_modpack_init(valid_pack, invalid_pack):
    """ModPack.__init__ behaves as described?"""

    mp = pack.ModPack(valid_pack)
    assert mp.data

    with pytest.raises(pack.ValidationError):
        mp = pack.ModPack(invalid_pack)


def test_modpack_create():
    """Does ModPack.create work as expected?"""

    gm = Game(id=42, name='Test', version='dev')
    mp = pack.ModPack.create(gm)

    assert mp.data['game']['name'] == gm.name
    assert mp.data['game']['version'] == gm.version


def test_modpack_load(pack_validator, valid_yaml, valid_pack, invalid_yaml):
    """Loading from stream works as advertised?"""

    mp = pack.ModPack.from_yaml(valid_yaml)
    assert mp.data == pack_validator.normalized(valid_pack)

    with pytest.raises(pack.ValidationError):
        mp = pack.ModPack.from_yaml(invalid_yaml)


def test_modpack_dump(pack_validator, valid_pack):
    """Dumping to stream works as advertised?"""

    stream = StringIO()
    mp = pack.ModPack(valid_pack)
    expect = pack_validator.normalized(deepcopy(valid_pack))

    mp.to_yaml(stream)
    print(stream.getvalue())
    data = yaml.load(stream.getvalue())

    assert data == expect



"""PyYAML adaptations and tweaks"""

import re
from collections.abc import Callable
from datetime import datetime
from enum import Enum
from functools import partial
from operator import attrgetter
from typing import Any

import yaml
from iso8601 import parse_date

# Load faster YAML implementation, if possible
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper


# Force ISO-8601 timestamps
def timestamp_representer(dumper: Dumper, date: datetime) -> yaml.Node:
    """Custom representer for datetime objects in YAML."""
    return dumper.represent_scalar('!!timestamp', date.isoformat())
Dumper.add_representer(datetime, timestamp_representer)


def timestamp_constructor(loader: Loader, node: yaml.Node) -> datetime:
    """Custom constructor for datetime objects from YAML."""
    value = loader.construct_scalar(node)
    return parse_date(value)
Loader.add_constructor('!!timestamp', timestamp_constructor)


# Decorator for nicer custom tags
class NodeType(Enum):
    SCALAR = 'construct_scalar', 'represent_scalar'
    SEQUENCE = 'construct_sequence', 'represent_sequence'
    MAPPING = 'construct_mapping', 'represent_mapping'

    def __init__(self: 'NodeType', constructor: str, representer: str) -> None:
        """Remember methodcaller instances for constructor and representer.

        Keyword arguments:
            constructor: Name of the constructor from YAML node.
            representer: Name of the representer to YAML node.
        """

        self.construct = attrgetter(constructor)
        self.represent = attrgetter(representer)


def tag(
    tag: str,
    type: NodeType = NodeType.SCALAR,
    pattern: str = None,
    *,
    Loader=Loader,
    Dumper=Dumper
):
    """Register a YAML tag for a class.

    The class should have defined to_yaml and from_yaml classmethods, with
    following signatures:

    .. py.classmethod:: to_yaml(instance: cls) -> Union[Any,Sequence,Mapping]
                        from_yaml(value: Union[Any,Sequence,Mapping]) -> cls

    Keyword arguments:
        tag: The tag name, should begin with '!'.
        type: Type of the YAML node. Returned and accepted values of to_yaml
            and from_yaml functions should be of compatible type.
        pattern: Optional regex pattern to implicitly resolve to tag.
        Loader: The loader to register to.
        Dumper: The dumper to register to.

    Raises:
        TypeError: When the decorated class does not have to_yaml or from_yaml
            methods.
    """

    def register(cls):
        """Closure registering the passed class."""

        # Test the presence and usability of the functions
        try:
            tested = cls.to_yaml, cls.from_yaml
        except AttributeError:
            raise TypeError('Missing YAML serialization method')

        if not all(isinstance(f, Callable) for f in tested):
            raise TypeError('YAML serialization method(s) are not callable')

        # Make conversion handlers
        def dump(dumper: Dumper, value: Any) -> yaml.Node:
            return type.represent(dumper)(tag, cls.to_yaml(value))

        def load(loader: Loader, node: yaml.Node) -> Any:
            return cls.from_yaml(type.construct(loader)(node))

        # Register conversions
        Dumper.add_representer(cls, dump)
        Loader.add_constructor(tag, load)

        if pattern is not None:
            regexp = re.compile(pattern)
            Dumper.add_implicit_resolver(tag, regexp, None)
            Loader.add_implicit_resolver(tag, regexp, None)

        return cls
    return register


# Provide load and dump functions with registered tweaks and sane defaults
load = partial(yaml.load, Loader=Loader)
dump = partial(yaml.dump, Dumper=Dumper, default_flow_style=False)



"""Package command line interface."""

import curses
from collections import ChainMap
from pathlib import Path
from typing import Mapping

import click

from . import _, pkgdir
from .curse import Game, Mod
from .pack import ModPack
from .proxy import Authorization
from .tui import select_mod
from .util import default_data_dir, yaml


def find_game(name: str, user_conf: Mapping = None) -> Mapping:
    """Find default parameters for a game.

    Keyword arguments:
        name: The game name to look for, case insensitive.
        user_conf: User-configured game defaults. This mapping should
            have the same structure as the package's game defaults
            configuration.

    Returns:
        Combined mapping of the parameter values, with user_conf
        taking precedence.
    """

    user_conf = dict() if user_conf is None else user_conf

    with (pkgdir/'games.yaml').open(encoding='utf-8') as stream:
        package_defaults = yaml.load(stream)

    return ChainMap(
        user_conf.get(name.lower(), dict()),
        package_defaults.get(name.lower(), dict()),
    )


def check_minecraft_dir(root: Path) -> None:
    """Checks if the directory is a suitable space for minecraft mods.

    Keyword Arguments:
        root: The checked dir – should be a root dir of minecraft profile.

    Raises:
        FileNotFoundError: When some expected file or directory is not found.
    """

    mods_dir = root / 'mods'

    if not mods_dir.is_dir():
        raise FileNotFoundError(str(mods_dir))


@click.group()
@click.version_option()
@click.option(
    '--game', '-g',
    type=click.STRING, default='Minecraft',
    help=_('Specify the game to mod.'),
)
@click.option(
    '--gamever', '-v',
    type=click.STRING, default=None,
    help=_('Specify the game version to mod.'),
)
@click.pass_context
def cli(ctx, game, gamever):
    """Minecraft Curse CLI client."""

    # Resolve game parameters
    game_params = find_game(game)
    if not game_params:
        raise SystemExit(_("Unknown game '{game}'").format_map(locals()))
    if gamever:
        game_params['version'] = gamever

    # Initialize terminal for querying
    curses.setupterm()

    # Add contextual values
    ctx.obj = {
        # Data directory
        'datadir': default_data_dir(),
        # Authentication file
        'authfile': default_data_dir() / 'token.yaml',
        # Game to work with
        'game': Game(name=game, **game_params),
    }


@cli.command()
@click.option(
    '--refresh', is_flag=True, default=False,
    # NOTE: Help for refresh flag
    help=_('Force refreshing of search data.')
)
@click.pass_obj
@click.argument('text', nargs=-1, type=str)
def search(ctx, refresh, text):
    """Search for TEXT in mods on CurseForge."""

    if not text:
        raise SystemExit(_('No text to search for!'))

    game = ctx['game']

    text = ' '.join(text)
    refresh = refresh or not game.have_fresh_data()

    if refresh:
        click.echo(_('Refreshing search data, please wait…'), err=True)
        game.refresh_data()

    found = Mod.search(game.database.session(), text)

    title = _('Search results for "{text}"').format(text=text)
    instructions = _(
        'Choose mod to open its project page, or press [q] to quit.'
    )

    chosen = select_mod(found, title, instructions)
    if chosen is not None:
        project_url_fmt = 'https://www.curseforge.com/projects/{mod.id}/'
        click.launch(project_url_fmt.format(mod=chosen))


@cli.command()
@click.option(
    '--user', '-u', prompt=_('Curse user name or email'),
    help=_('Curse user name or email')+'.',
)
@click.password_option(
    help=_('Curse password')+'.',
)
@click.pass_obj
def auth(ctx, user, password):
    """Authenticate user in Curse network."""

    token = Authorization.login(user, password)

    with ctx['authfile'].open(mode='w', encoding='utf-8') as file:
        token.dump(file)


@cli.command()
@click.option(
    '--profile', '-p', type=click.Path(exists=True, file_okay=False),
    default='.',
    help=_('Root profile directory')+'.',
)
@click.pass_obj
def new(ctx, profile):
    """Create a new modpack for Minecraft VERSION."""

    game = ctx['game']
    profile = Path(str(profile))

    try:
        check_minecraft_dir(profile)
    except FileNotFoundError as err:
        msg = _('Profile directory is not valid: Missing {!s}').format(err)
        raise SystemExit(msg) from None

    modpack_file = profile / 'modpack.yaml'
    with modpack_file.open(mode='w', encoding='utf-8') as stream:
        ModPack.create(game).to_yaml(stream)
